@@ -8,7 +8,7 @@ package Devel::MAT::Tool;
 use strict;
 use warnings;
 
-our $VERSION = '0.27';
+our $VERSION = '0.28';
 
 use List::Util qw( any );
 
@@ -57,7 +57,11 @@ sub get_sv
          $sv = $self->df->$addr;
       }
       else {
-         $addr = hex $addr if $addr =~ m/^0x/;
+         $addr = do {
+            no warnings 'portable';
+            hex $addr;
+         } if $addr =~ m/^0x/;
+
          do { no warnings 'numeric'; $addr eq $addr+0 } or
             die "Expected numerical SV address\n";
 
