@@ -9,7 +9,7 @@ use strict;
 use warnings;
 use base qw( Devel::MAT::Tool );
 
-our $VERSION = '0.27';
+our $VERSION = '0.28';
 
 use constant CMD => "io";
 
@@ -74,8 +74,10 @@ sub run_cmd_list
    foreach ( sort _by_fileno keys %ios ) {
       my $sv = $ios{$_};
 
-      my $len = Devel::MAT::Cmd->print_sv( $sv );
-      Devel::MAT::Cmd->printf( "%s %-8s %-8s\n", " " x ( 30 - $len ), $sv->ifileno // "-", $sv->ofileno // "-" );
+      Devel::MAT::Cmd->printf( "%-30s %-8s %-8s\n", 
+         Devel::MAT::Cmd->format_sv( $sv ),
+         $sv->ifileno // "-",
+         $sv->ofileno // "-" );
    }
 }
 
@@ -101,8 +103,10 @@ sub run_cmd_find
 
       next unless $sv->ifileno == $num or $sv->ofileno == $num;
 
-      my $len = Devel::MAT::Cmd->print_sv( $sv );
-      Devel::MAT::Cmd->printf( "%s %-8s %-8s\n", " " x ( 30 - $len ), $sv->ifileno // "-", $sv->ofileno // "-" );
+      Devel::MAT::Cmd->printf( "%-30s %-8s %-8s\n",
+         Devel::MAT::Cmd->format_sv( $sv ),
+         $sv->ifileno // "-",
+         $sv->ofileno // "-" );
    }
 }
 
