@@ -10,7 +10,7 @@ use warnings;
 use base qw( Devel::MAT::Tool );
 use utf8;
 
-our $VERSION = '0.27';
+our $VERSION = '0.28';
 
 use Getopt::Long qw( GetOptionsFromArray );
 use List::Util qw( pairs );
@@ -65,13 +65,15 @@ sub walk_graph
       Devel::MAT::Cmd->printf( "$indent└─already found " );
 
       if( defined( my $id = $id_for{$node->addr} ) ) {
-         Devel::MAT::Cmd->printf( "as " );
-         Devel::MAT::Cmd->print_note( "*$id" );
+         Devel::MAT::Cmd->printf( "as %s\n",
+            Devel::MAT::Cmd->format_note( "*$id" ),
+         );
       }
       else {
-         Devel::MAT::Cmd->print_note( "circularly" );
+         Devel::MAT::Cmd->printf( "%s\n",
+            Devel::MAT::Cmd->format_note( "circularly" ),
+         );
       }
-      Devel::MAT::Cmd->printf( "\n" );
       return;
    }
 
@@ -91,21 +93,24 @@ sub walk_graph
       Devel::MAT::Cmd->printf(
          $indent . ( $is_final ? "└─" : "├─" ) );
 
-      Devel::MAT::Cmd->print_note(
-         "[" . $ref->strength . "]", 1 ) if $ref->strength ne "strong";
+      Devel::MAT::Cmd->printf( "%s",
+         Devel::MAT::Cmd->format_note( "[" . $ref->strength . "]", 1 ),
+      ) if $ref->strength ne "strong";
 
       my $ref_id;
       if( $refnode->edges_out > 1 and not $refnode->roots and not $id_for{$refnode->addr} ) {
          $ref_id = $id_for{$refnode->addr} = $next_id++;
       }
 
-      Devel::MAT::Cmd->printf( "%s of ", $ref->name );
-      Devel::MAT::Cmd->print_sv( $refnode->sv );
-      Devel::MAT::Cmd->printf( ", which is" );
+      Devel::MAT::Cmd->printf( "%s of %s, which is",
+         $ref->name,
+         Devel::MAT::Cmd->format_sv( $refnode->sv ),
+      );
 
       if( $ref_id ) {
-         Devel::MAT::Cmd->printf( " " );
-         Devel::MAT::Cmd->print_note( "(*$ref_id)" );
+         Devel::MAT::Cmd->printf( " %s",
+            Devel::MAT::Cmd->format_note( "(*$ref_id)" ),
+         );
       }
 
       Devel::MAT::Cmd->printf( ":\n" );
@@ -194,8 +199,9 @@ sub run_cmd
 
    $self->pmat->load_tool( "Inrefs", progress => $self->{progress} );
 
-   Devel::MAT::Cmd->print_sv( $sv );
-   Devel::MAT::Cmd->printf( " is:\n" );
+   Devel::MAT::Cmd->printf( "%s is:\n",
+      Devel::MAT::Cmd->format_sv( $sv ),
+   );
 
    walk_graph( $self->pmat->inref_graph( $sv,
       depth => $DEPTH,
