@@ -1,16 +1,23 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2013-2016 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2013-2017 -- leonerd@leonerd.org.uk
 
 package Devel::MAT::Tool::Inrefs;
 
 use strict;
 use warnings;
 
-our $VERSION = '0.27';
+our $VERSION = '0.28';
 
-use List::Util qw( pairmap pairs );
+use List::Util qw( pairs );
+
+my %STRENGTH_TO_IDX = (
+   strong   => 0,
+   weak     => 1,
+   indirect => 2,
+   inferred => 3,
+);
 
 =head1 NAME
 
@@ -35,14 +42,6 @@ sub new
    return $class;
 }
 
-my %PREFIX_FROM_STRENGTH = (
-   strong   => "+",
-   weak     => "-",
-   indirect => ";",
-   inferred => ".",
-);
-my %STRENGTH_FROM_PREFIX = reverse %PREFIX_FROM_STRENGTH;
-
 sub patch_inrefs
 {
    my $self = shift;
@@ -50,31 +49,26 @@ sub patch_inrefs
 
    my $progress = $args{progress};
 
+   # Performance optimisation since we don't care about the 'name' of any of
+   #   these outrefs
+   no warnings 'redefine';
+   local *Devel::MAT::Cmd::format_value = sub { "" };
+   local *Devel::MAT::Cmd::format_note  = sub { "" };
+
    my $heap_total = scalar $df->heap;
    my $count = 0;
    foreach my $sv ( $df->heap ) {
-      foreach my $ref ( $sv->_outrefs_matching ) {
+      foreach my $ref ( $sv->outrefs ) {
          my $refsv = $ref->sv;
-         my $prefixname = $PREFIX_FROM_STRENGTH{$ref->strength} . $ref->name;
-         push @{ $refsv->{tool_inrefs} }, $prefixname, $sv->addr if !$refsv->immortal;
+         push @{ $refsv->{tool_inrefs}[ $STRENGTH_TO_IDX{ $ref->strength } ] }, $sv->addr if !$refsv->immortal;
       }
 
       $count++;
       $progress->( sprintf "Patching refs in %d of %d (%.2f%%)",
-         $count, $heap_total, 100*$count / $heap_total ) if $progress and ($count % 2000) == 0
+         $count, $heap_total, 100*$count / $heap_total ) if $progress and ($count % 10000) == 0
    }
 
    $progress->() if $progress;
-
-   foreach ( pairs $df->_roots ) {
-      my ( $name, $sv ) = @$_;
-      push @{ $sv->{tool_inrefs} }, $name, undef if defined $sv;
-   }
-
-   foreach my $addr ( @{ $df->{stack_at} } ) { # TODO
-      my $sv = $df->sv_at( $addr ) or next;
-      push @{ $sv->{tool_inrefs} }, "a value on the stack", undef;
-   }
 }
 
 =head1 SV METHODS
@@ -116,43 +110,52 @@ C<outrefs_*> methods.
 sub Devel::MAT::SV::_inrefs
 {
    my $self = shift;
-   my ( $match ) = @_;
+   my ( $just_svs, @strengths ) = @_;
 
    $self->{tool_inrefs} ||= [];
 
    my $df = $self->df;
    my @inrefs;
-   foreach ( pairs @{ $self->{tool_inrefs} } ) {
-      my ( $name, $addr ) = @$_;
-
-      if( wantarray and $addr and $addr =~ m/^\d+$/ ) {
+   foreach my $strength ( @strengths ) {
+      my %seen;
+      foreach my $addr ( @{ $self->{tool_inrefs}[ $STRENGTH_TO_IDX{$strength} ] // [] } ) {
          my $sv = $df->sv_at( $addr );
-         if( $sv ) {
-            push @inrefs, $name, $sv if $name =~ $match;
+
+         if( $just_svs ) {
+            push @inrefs, $sv;
          }
          else {
-            warn "Unable to find SV at $_ for $sv inref\n";
+            $seen{$addr}++ and next;
+
+            push @inrefs, Devel::MAT::SV::Reference( $_->name, $_->strength, $sv )
+               for grep { $_->strength eq $strength and $_->sv == $self } $sv->outrefs;
          }
       }
-      else {
-         push @inrefs, $name => undef if $name =~ $match;
-      }
    }
 
-   return @inrefs / 2 if !wantarray;
+   foreach ( pairs $df->_roots ) {
+      my ( $name, $sv ) = @$_;
+      push @inrefs, $just_svs ? undef : Devel::MAT::SV::Reference( $name, strong => undef )
+         if defined $sv and $sv == $self;
+   }
 
-   return pairmap {
-      my $prefix = substr( $a, 0, 1, "" );
-      Devel::MAT::SV::Reference( $a, $STRENGTH_FROM_PREFIX{$prefix}, $b );
-   } @inrefs;
+   foreach my $addr ( @{ $df->{stack_at} } ) { # TODO
+      next unless $addr == $self->addr;
+
+      push @inrefs, $just_svs ? undef :
+         Devel::MAT::SV::Reference( "a value on the stack", strong => undef );
+   }
+
+   return @inrefs;
 }
 
-sub Devel::MAT::SV::inrefs          { shift->_inrefs( qr/^./    ) }
-sub Devel::MAT::SV::inrefs_strong   { shift->_inrefs( qr/^\+/   ) }
-sub Devel::MAT::SV::inrefs_weak     { shift->_inrefs( qr/^-/    ) }
-sub Devel::MAT::SV::inrefs_direct   { shift->_inrefs( qr/^[+-]/ ) }
-sub Devel::MAT::SV::inrefs_indirect { shift->_inrefs( qr/^;/    ) }
-sub Devel::MAT::SV::inrefs_inferred { shift->_inrefs( qr/^\./   ) }
+sub Devel::MAT::SV::inrefs          { shift->_inrefs( !wantarray, qw( strong weak indirect inferred )) }
+
+sub Devel::MAT::SV::inrefs_strong   { shift->_inrefs( !wantarray, qw( strong      )) }
+sub Devel::MAT::SV::inrefs_weak     { shift->_inrefs( !wantarray, qw( weak        )) }
+sub Devel::MAT::SV::inrefs_direct   { shift->_inrefs( !wantarray, qw( strong weak )) }
+sub Devel::MAT::SV::inrefs_indirect { shift->_inrefs( !wantarray, qw( indirect    )) }
+sub Devel::MAT::SV::inrefs_inferred { shift->_inrefs( !wantarray, qw( inferred    )) }
 
 =head1 AUTHOR
 
