@@ -11,7 +11,7 @@ use base qw( Devel::MAT::Tool );
 
 use List::Util qw( max );
 
-our $VERSION = '0.27';
+our $VERSION = '0.28';
 
 use constant CMD => "roots";
 
@@ -49,9 +49,9 @@ sub run_cmd
    foreach my $name ( sort keys %roots ) {
       my $sv = $roots{$name} or next;  # Not all root SVs are defined
 
-      Devel::MAT::Cmd->printf( "%-*s: ", $namelen, $name );
-      Devel::MAT::Cmd->print_sv( $sv );
-      Devel::MAT::Cmd->printf( "\n" );
+      Devel::MAT::Cmd->printf( "%-*s: %s\n",
+         $namelen, $name,
+         Devel::MAT::Cmd->format_sv( $sv ) );
    }
 }
 
