@@ -9,7 +9,7 @@ use strict;
 use warnings;
 use base qw( Devel::MAT::Tool );
 
-our $VERSION = '0.27';
+our $VERSION = '0.28';
 
 use List::Util qw( max );
 
@@ -68,9 +68,23 @@ sub run_cmd
    my ( $addr ) = @_;
    my $sv = $self->get_sv( $addr );
 
-   Devel::MAT::Cmd->print_sv( $sv );
+   Devel::MAT::Cmd->printf( "%s with refcount %d\n",
+      Devel::MAT::Cmd->format_sv( $sv ),
+      $sv->refcnt,
+   );
 
-   Devel::MAT::Cmd->printf( " with refcount %d\n", $sv->refcnt );
+   my $size = $sv->size;
+   if( $size < 1024 ) {
+      Devel::MAT::Cmd->printf( "  size %d bytes\n",
+         $size,
+      );
+   }
+   else {
+      Devel::MAT::Cmd->printf( "  size %s (%d bytes)\n",
+         Devel::MAT::Cmd->format_bytes( $size ),
+         $size,
+      );
+   }
 
    if( my $stash = $sv->blessed ) {
       Devel::MAT::Cmd->printf( "  blessed as %s\n", $stash->stashname );
@@ -83,11 +97,13 @@ sub run_cmd
 
 sub say_with_sv
 {
-   my ( $str, $sv ) = @_;
+   my ( $str, @args ) = @_;
+   my $sv = pop @args;
 
-   Devel::MAT::Cmd->printf( "%s", $str );
-   Devel::MAT::Cmd->print_sv( $sv );
-   Devel::MAT::Cmd->printf( "\n" );
+   Devel::MAT::Cmd->printf( $str . "%s\n",
+      @args,
+      Devel::MAT::Cmd->format_sv( $sv ),
+   );
 }
 
 sub show_GLOB
@@ -142,7 +158,9 @@ sub show_ARRAY
       }
 
       if( $elems[$idx] ) {
-         say_with_sv( "  [$idx]=", $elems[$idx] );
+         say_with_sv( "  %s=",
+            Devel::MAT::Cmd->format_value( $idx, index => 1 ), $elems[$idx]
+         );
       }
       else {
          Devel::MAT::Cmd->printf( "  [%d]=NULL\n", $idx );
@@ -171,19 +189,14 @@ sub show_HASH
          last;
       }
 
-      my $strkey = $key;
-      if( $strkey !~ m/^[a-zA-Z_][a-zA-Z0-9_]*$/ ) {
-         $strkey =~ s(([\\"\$\@]))(\\$1)g;
-         $strkey =~ s{([\x00-\x1f])}{sprintf "\\x%02x", ord $1}eg;
-         $strkey = qq("$strkey");
-      }
+      my $fmtkey = Devel::MAT::Cmd->format_value( $key, key => 1 );
 
       my $sv = $hv->value( $key );
       if( $sv ) {
-         say_with_sv( "  {$strkey}=", $sv );
+         say_with_sv( "  %s=", $fmtkey, $sv );
       }
       else {
-         Devel::MAT::Cmd->printf( "  {%s}=NULL\n", $strkey );
+         Devel::MAT::Cmd->printf( "  %s=NULL\n", $fmtkey );
       }
 
       $count++;
@@ -224,7 +237,7 @@ sub show_CODE
 
    if( my @globs = $cv->globrefs ) {
       Devel::MAT::Cmd->printf( "Referenced globs:\n  " );
-      Devel::MAT::Cmd->print_sv( $_ ), Devel::MAT::Cmd->printf( ", " ) for @globs;
+      Devel::MAT::Cmd->printf( "%s, ", Devel::MAT::Cmd->format_sv( $_ ) ) for @globs;
       Devel::MAT::Cmd->printf( "\n" );
    }
 }
@@ -256,14 +269,18 @@ sub show_PAD
    foreach my $padix ( 1 .. $#elems ) {
       my $sv = $elems[$padix];
       if( $padnames[$padix] ) {
-         Devel::MAT::Cmd->printf( "  [%3d/%*s]=", $padix, $maxname, $padnames[$padix] );
-         $sv ? Devel::MAT::Cmd->print_sv( $sv ) : Devel::MAT::Cmd->printf( "NULL" );
-         Devel::MAT::Cmd->printf( "\n" );
+         Devel::MAT::Cmd->printf( "  [%3d/%*s]=%s\n",
+            $padix,
+            $maxname, $padnames[$padix],
+            ( $sv ? Devel::MAT::Cmd->format_sv( $sv ) : "NULL" ),
+         );
       }
       else {
-         Devel::MAT::Cmd->printf( "  [%3d %*s]=", $padix, $maxname, $padtype{$padix} // "" );
-         $sv ? Devel::MAT::Cmd->print_sv( $sv ) : Devel::MAT::Cmd->printf( "NULL" );
-         Devel::MAT::Cmd->printf( "\n" );
+         Devel::MAT::Cmd->printf( "  [%3d %*s]=%s\n",
+            $padix,
+            $maxname, $padtype{$padix} // "",
+            ( $sv ? Devel::MAT::Cmd->format_sv( $sv ) : "NULL" ),
+         );
       }
    }
 }
