@@ -9,7 +9,7 @@ use strict;
 use warnings;
 use base qw( Devel::MAT::Tool );
 
-our $VERSION = '0.27';
+our $VERSION = '0.28';
 
 use constant CMD => "symbols";
 
@@ -78,9 +78,10 @@ sub _show_symbol
 {
    my ( $name, $sv ) = @_;
 
-   Devel::MAT::Cmd->printf( "%s at ", $name );
-   Devel::MAT::Cmd->print_sv( $sv );
-   Devel::MAT::Cmd->printf( "\n" );
+   Devel::MAT::Cmd->printf( "%s at %s\n",
+      $name,
+      Devel::MAT::Cmd->format_sv( $sv ),
+   );
 }
 
 sub run_cmd
